@@ -15,20 +15,21 @@
  * limitations under the License.
  */
 package org.apache.commons.net.ftp.parser;
-import junit.framework.TestCase;
-
 import java.text.SimpleDateFormat;
 import java.util.Calendar;
 import java.util.Locale;
+
 import org.apache.commons.net.ftp.FTPFile;
 import org.apache.commons.net.ftp.FTPFileEntryParser;
 
+import junit.framework.TestCase;
+
 /**
  */
 public abstract class FTPParseTestFramework extends TestCase
 {
-    private FTPFileEntryParser parser = null;
-    protected SimpleDateFormat df = null;
+    private FTPFileEntryParser parser;
+    protected SimpleDateFormat df;
 
     /**
      * @see junit.framework.TestCase#TestCase(String)
@@ -38,7 +39,7 @@ public abstract class FTPParseTestFramework extends TestCase
         super(name);
     }
 
-    public void testBadListing() throws Exception
+    public void testBadListing()
     {
 
         final String[] badsamples = getBadListing();
@@ -53,7 +54,7 @@ public abstract class FTPParseTestFramework extends TestCase
         }
     }
 
-    public void testGoodListing() throws Exception
+    public void testGoodListing()
     {
 
         final String[] goodsamples = getGoodListing();
