@@ -22,8 +22,6 @@ import org.apache.commons.net.ftp.FTPFile;
 import org.apache.commons.net.ftp.FTPFileEntryParser;
 
 /**
- * @author <a href="mailto:scohen@apache.org">Steve Cohen</a>
- * @version $Id: UnixFTPEntryParserTest.java 1489533 2013-06-04 17:49:00Z sebb $
  */
 public class UnixFTPEntryParserTest extends FTPParseTestFramework {
 
@@ -70,96 +68,84 @@ public class UnixFTPEntryParserTest extends FTPParseTestFramework {
             "drwxrwx---+ 23 500     500    0 Jan 10 13:09 testACL",
             "-rw-r--r--   1 1        3518644 May 25 12:12 std",
             "lrwxrwxrwx   1 neeme neeme             23 Mar  2 18:06 macros -> ./../../global/macros/.",
-            "-rw-r--r--   1 ftp      group with spaces in it as allowed in cygwin see bug 38634   83853 Jan 22  2001 zxJDBC-1.2.4.tar.gz",
+            "-rw-r--r--   1 ftp      group with spaces in it as allowed in cygwin see bug 38634" +
+                         "   83853 Jan 22  2001 zxJDBC-1.2.4.tar.gz",
                                                                                    // Bug 38634 => NET-16
             "crw-r----- 1 root kmem 0, 27 Jan 30 11:42 kmem",  //FreeBSD device
             "crw-------   1 root     sys      109,767 Jul  2  2004 pci@1c,600000:devctl", //Solaris device
             "-rwxrwx---   1 ftp      ftp-admin 816026400 Oct  5  2008 bloplab 7 cd1.img", // NET-294
 
+            // https://mail-archives.apache.org/mod_mbox/commons-dev/200408.mbox/%3c4122F3C1.9090402@tanukisoftware.com%3e
+            "-rw-r--r-- 1 1 3518644 May 25 12:12 std",
+            "-rw-rw---- 1 ep1adm sapsys 0 6\u6708 3\u65e5 2003\u5e74 \u8a66\u9a13\u30d5\u30a1\u30a4\u30eb.csv",
+            "-rw-rw---- 1 ep1adm sapsys 0 8\u6708 17\u65e5 20:10 caerrinf",
 
-        };
+    };
 
-    /**
-     * @see junit.framework.TestCase#TestCase(String)
-     */
-    public UnixFTPEntryParserTest(String name) {
+    public UnixFTPEntryParserTest(final String name) {
         super(name);
     }
 
-    /**
-     * @see org.apache.commons.net.ftp.parser.FTPParseTestFramework#getBadListing()
-     */
     @Override
     protected String[] getBadListing() {
-        return (badsamples);
+        return badsamples;
     }
 
-    /**
-     * @see org.apache.commons.net.ftp.parser.FTPParseTestFramework#getGoodListing()
-     */
     @Override
     protected String[] getGoodListing() {
-        return (goodsamples);
+        return goodsamples;
     }
 
-    /**
-     */
     public void testNumericDateFormat()
     {
-        String testNumericDF =
+        final String testNumericDF =
             "-rw-r-----   1 neeme neeme   346 2005-04-08 11:22 services.vsp";
-        String testNumericDF2 =
+        final String testNumericDF2 =
             "lrwxrwxrwx   1 neeme neeme    23 2005-03-02 18:06 macros -> ./../../global/macros/.";
 
-        UnixFTPEntryParser parser =
+        final UnixFTPEntryParser parser =
             new UnixFTPEntryParser(UnixFTPEntryParser.NUMERIC_DATE_CONFIG);
 
-        FTPFile f = parser.parseFTPEntry(testNumericDF);
-        assertNotNull("Failed to parse " + testNumericDF,
-                      f);
+        final FTPFile f = parser.parseFTPEntry(testNumericDF);
+        assertNotNull("Failed to parse " + testNumericDF, f);
 
 
-        Calendar cal = Calendar.getInstance();
+        final Calendar cal = Calendar.getInstance();
         cal.clear();
         cal.set(Calendar.YEAR, 2005);
         cal.set(Calendar.MONTH, Calendar.APRIL);
 
-        cal.set(Calendar.DATE, 8);
+        cal.set(Calendar.DAY_OF_MONTH, 8);
         cal.set(Calendar.HOUR_OF_DAY, 11);
         cal.set(Calendar.MINUTE, 22);
         assertEquals(cal.getTime(), f.getTimestamp().getTime());
 
-        FTPFile f2 = parser.parseFTPEntry(testNumericDF2);
-        assertNotNull("Failed to parse " + testNumericDF2,
-                      f2);
+        final FTPFile f2 = parser.parseFTPEntry(testNumericDF2);
+        assertNotNull("Failed to parse " + testNumericDF2, f2);
         assertEquals("symbolic link", "./../../global/macros/.", f2.getLink());
 
     }
 
-
-    /**
-     * @see org.apache.commons.net.ftp.parser.FTPParseTestFramework#getParser()
-     */
     @Override
     protected FTPFileEntryParser getParser() {
-        return (new UnixFTPEntryParser());
+        return new UnixFTPEntryParser();
     }
 
     public void testOwnerNameWithSpaces() {
-        FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 john smith     group         4096 Mar  2 15:13 zxbox");
+        final FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 john smith     group         4096 Mar  2 15:13 zxbox");
         assertNotNull(f);
         assertEquals("john smith", f.getUser());
     }
 
     public void testOwnerAndGroupNameWithSpaces() {
-        FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 john smith     test group         4096 Mar  2 15:13 zxbox");
+        final FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 john smith     test group         4096 Mar  2 15:13 zxbox");
         assertNotNull(f);
         assertEquals("john smith", f.getUser());
         assertEquals("test group", f.getGroup());
     }
 
     public void testNET294() {
-        FTPFile f = getParser().parseFTPEntry(
+        final FTPFile f = getParser().parseFTPEntry(
                 "-rwxrwx---   1 ftp      ftp-admin 816026400 Oct  5  2008 bloplab 7 cd1.img");
         assertNotNull(f);
         assertEquals("ftp", f.getUser());
@@ -171,44 +157,64 @@ public class UnixFTPEntryParserTest extends FTPParseTestFramework {
     }
 
     public void testGroupNameWithSpaces() {
-        FTPFile f = getParser().parseFTPEntry("drwx------ 4 maxm Domain Users 512 Oct 2 10:59 .metadata");
+        final FTPFile f = getParser().parseFTPEntry("drwx------ 4 maxm Domain Users 512 Oct 2 10:59 .metadata");
         assertNotNull(f);
         assertEquals("maxm", f.getUser());
         assertEquals("Domain Users", f.getGroup());
     }
 
     public void testTrailingSpaces() {
-        FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 john smith     group         4096 Mar  2 15:13 zxbox     ");
+        final FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 john smith     group         4096 Mar  2 15:13 zxbox     ");
         assertNotNull(f);
-        assertEquals(f.getName(), "zxbox     ");
+        assertEquals("zxbox     ", f.getName());
+    }
+
+    public void testLeadingSpacesDefault() { // the default has been changed to keep spaces
+        final FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 john smith     group         4096 Mar  2 15:13   zxbox");
+        assertNotNull(f);
+        assertEquals("  zxbox", f.getName() ); // leading spaces retained
+    }
+
+    public void testLeadingSpacesNET566() { // check new behavior
+        final FTPFile f = new UnixFTPEntryParser(null, false).parseFTPEntry(
+            "drwxr-xr-x   2 john smith     group         4096 Mar  2 15:13   zxbox");
+        assertNotNull(f);
+        assertEquals("  zxbox", f.getName() ); // leading spaces retained
+    }
+
+    public void testTrimLeadingSpacesNET566() { // check can trim spaces as before
+        final FTPFile f = new UnixFTPEntryParser(null, true).parseFTPEntry(
+            "drwxr-xr-x   2 john smith     group         4096 Mar  2 15:13   zxbox");
+        assertNotNull(f);
+        assertEquals("zxbox", f.getName() ); // leading spaces trimmed
     }
 
     public void testNameWIthPunctuation() {
-        FTPFile f = getParser().parseFTPEntry("drwx------ 4 maxm Domain Users 512 Oct 2 10:59 abc(test)123.pdf");
+        final FTPFile f = getParser().parseFTPEntry("drwx------ 4 maxm Domain Users 512 Oct 2 10:59 abc(test)123.pdf");
         assertNotNull(f);
-        assertEquals(f.getName(), "abc(test)123.pdf");
+        assertEquals("abc(test)123.pdf", f.getName());
     }
 
     public void testNoSpacesBeforeFileSize() {
-        FTPFile f = getParser().parseFTPEntry("drwxr-x---+1464 chrism   chrism     41472 Feb 25 13:17 20090225");
+        final FTPFile f = getParser().parseFTPEntry("drwxr-x---+1464 chrism   chrism     41472 Feb 25 13:17 20090225");
         assertNotNull(f);
-        assertEquals(f.getSize(), 41472);
+        assertEquals(41472, f.getSize());
         assertEquals(f.getType(), FTPFile.DIRECTORY_TYPE);
-        assertEquals(f.getUser(), "chrism");
-        assertEquals(f.getGroup(), "chrism");
-        assertEquals(f.getHardLinkCount(), 1464);
+        assertEquals("chrism", f.getUser());
+        assertEquals("chrism", f.getGroup());
+        assertEquals(1464, f.getHardLinkCount());
     }
 
     public void testCorrectGroupNameParsing() {
-        FTPFile f = getParser().parseFTPEntry("-rw-r--r--   1 ftpuser  ftpusers 12414535 Mar 17 11:07 test 1999 abc.pdf");
+        final FTPFile f = getParser().parseFTPEntry("-rw-r--r--   1 ftpuser  ftpusers 12414535 Mar 17 11:07 test 1999 abc.pdf");
         assertNotNull(f);
-        assertEquals(f.getHardLinkCount(), 1);
-        assertEquals(f.getUser(), "ftpuser");
-        assertEquals(f.getGroup(), "ftpusers");
-        assertEquals(f.getSize(), 12414535);
-        assertEquals(f.getName(), "test 1999 abc.pdf");
+        assertEquals(1, f.getHardLinkCount());
+        assertEquals("ftpuser", f.getUser());
+        assertEquals("ftpusers", f.getGroup());
+        assertEquals(12414535, f.getSize());
+        assertEquals("test 1999 abc.pdf", f.getName());
 
-        Calendar cal = Calendar.getInstance();
+        final Calendar cal = Calendar.getInstance();
         cal.set(Calendar.MONTH, Calendar.MARCH);
         cal.set(Calendar.DAY_OF_MONTH, 17);
         cal.set(Calendar.HOUR_OF_DAY, 11);
@@ -224,19 +230,16 @@ public class UnixFTPEntryParserTest extends FTPParseTestFramework {
     }
 
     public void testFilenamesWithEmbeddedNumbers() {
-        FTPFile f = getParser().parseFTPEntry("-rw-rw-rw-   1 user group 5840 Mar 19 09:34 123 456 abc.csv");
-        assertEquals(f.getName(), "123 456 abc.csv");
-        assertEquals(f.getSize(), 5840);
-        assertEquals(f.getUser(), "user");
-        assertEquals(f.getGroup(), "group");
+        final FTPFile f = getParser().parseFTPEntry("-rw-rw-rw-   1 user group 5840 Mar 19 09:34 123 456 abc.csv");
+        assertEquals("123 456 abc.csv", f.getName());
+        assertEquals(5840, f.getSize());
+        assertEquals("user", f.getUser());
+        assertEquals("group", f.getGroup());
     }
 
-    /**
-     * @see org.apache.commons.net.ftp.parser.FTPParseTestFramework#testParseFieldsOnDirectory()
-     */
     @Override
     public void testParseFieldsOnDirectory() throws Exception {
-        FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 user     group         4096 Mar  2 15:13 zxbox");
+        final FTPFile f = getParser().parseFTPEntry("drwxr-xr-x   2 user     group         4096 Mar  2 15:13 zxbox");
         assertNotNull("Could not parse entry.", f);
         assertTrue("Should have been a directory.", f.isDirectory());
         checkPermissions(f);
@@ -246,31 +249,35 @@ public class UnixFTPEntryParserTest extends FTPParseTestFramework {
         assertEquals("zxbox", f.getName());
         assertEquals(4096, f.getSize());
 
-        Calendar cal = Calendar.getInstance();
+        final Calendar cal = Calendar.getInstance();
         cal.set(Calendar.MONTH, Calendar.MARCH);
 
-        cal.set(Calendar.DATE, 1);
+        cal.set(Calendar.DAY_OF_MONTH, 1);
         cal.set(Calendar.HOUR_OF_DAY, 0);
         cal.set(Calendar.MINUTE, 0);
         cal.set(Calendar.SECOND, 0);
         if (f.getTimestamp().getTime().before(cal.getTime())) {
             cal.add(Calendar.YEAR, -1);
         }
-        cal.set(Calendar.DATE, 2);
+        cal.set(Calendar.DAY_OF_MONTH, 2);
         cal.set(Calendar.HOUR_OF_DAY, 15);
         cal.set(Calendar.MINUTE, 13);
 
-        assertEquals(df.format(cal.getTime()), df.format(f.getTimestamp()
-                .getTime()));
+        assertEquals(df.format(cal.getTime()), df.format(f.getTimestamp().getTime()));
     }
 
 
-    /**
-     * Method checkPermissions.
-     * Verify that the persmissions were properly set.
-     * @param f
-     */
-    private void checkPermissions(FTPFile f) {
+    @Override
+    public void testRecentPrecision() {
+        testPrecision("drwxr-xr-x   2 user     group         4096 Mar  2 15:13 zxbox", CalendarUnit.MINUTE);
+    }
+
+    @Override
+    public void testDefaultPrecision() {
+        testPrecision("drwxr-xr-x   2 user     group         4096 Mar  2 2014 zxbox", CalendarUnit.DAY_OF_MONTH);
+    }
+
+    private void checkPermissions(final FTPFile f) {
         assertTrue("Should have user read permission.", f.hasPermission(
                 FTPFile.USER_ACCESS, FTPFile.READ_PERMISSION));
         assertTrue("Should have user write permission.", f.hasPermission(
@@ -291,12 +298,9 @@ public class UnixFTPEntryParserTest extends FTPParseTestFramework {
                 FTPFile.WORLD_ACCESS, FTPFile.EXECUTE_PERMISSION));
     }
 
-    /**
-     * @see org.apache.commons.net.ftp.parser.FTPParseTestFramework#testParseFieldsOnFile()
-     */
     @Override
     public void testParseFieldsOnFile() throws Exception {
-        FTPFile f = getParser()
+        final FTPFile f = getParser()
                 .parseFTPEntry(
                         "-rwxr-xr-x   2 user     my group 500        5000000000 Mar  2 15:13 zxbox");
         assertNotNull("Could not parse entry.", f);
@@ -308,37 +312,86 @@ public class UnixFTPEntryParserTest extends FTPParseTestFramework {
         assertEquals("zxbox", f.getName());
         assertEquals(5000000000L, f.getSize());
 
-        Calendar cal = Calendar.getInstance();
+        final Calendar cal = Calendar.getInstance();
         cal.set(Calendar.MONTH, Calendar.MARCH);
 
-        cal.set(Calendar.DATE, 1);
+        cal.set(Calendar.DAY_OF_MONTH, 1);
         cal.set(Calendar.HOUR_OF_DAY, 0);
         cal.set(Calendar.MINUTE, 0);
         cal.set(Calendar.SECOND, 0);
         if (f.getTimestamp().getTime().before(cal.getTime())) {
             cal.add(Calendar.YEAR, -1);
         }
-        cal.set(Calendar.DATE, 2);
+        cal.set(Calendar.DAY_OF_MONTH, 2);
         cal.set(Calendar.HOUR_OF_DAY, 15);
         cal.set(Calendar.MINUTE, 13);
         assertEquals(df.format(cal.getTime()), df.format(f.getTimestamp().getTime()));
     }
 
-    /*
-     * @param test
-     * @param f
-     */
+    // https://mail-archives.apache.org/mod_mbox/commons-dev/200408.mbox/%3c4122F3C1.9090402@tanukisoftware.com%3e
+    public void testParseFieldsOnFileJapaneseTime()
+    {
+        final FTPFile f = getParser().parseFTPEntry("-rwxr-xr-x 2 user group 4096 3\u6708 2\u65e5 15:13 zxbox");
+        assertNotNull("Could not parse entry.", f);
+        assertTrue("Should have been a file.", f.isFile());
+        checkPermissions(f);
+        assertEquals(2, f.getHardLinkCount());
+        assertEquals("user", f.getUser());
+        assertEquals("group", f.getGroup());
+        assertEquals("zxbox", f.getName());
+        assertEquals(4096, f.getSize());
+
+        assertNotNull("Timestamp not null", f.getTimestamp());
+        final Calendar cal = Calendar.getInstance();
+        cal.set(Calendar.MONTH, Calendar.MARCH);
+        cal.set(Calendar.DATE,1);
+        cal.set(Calendar.HOUR_OF_DAY, 0);
+        cal.set(Calendar.MINUTE, 0);
+        cal.set(Calendar.SECOND, 0);
+        if (f.getTimestamp().getTime().before(cal.getTime())) {
+            cal.add(Calendar.YEAR, -1);
+        }
+        cal.set(Calendar.DATE,2);
+        cal.set(Calendar.HOUR_OF_DAY, 15);
+        cal.set(Calendar.MINUTE, 13);
+        assertEquals(df.format(cal.getTime()), df.format(f.getTimestamp().getTime()));
+    }
+
+ // https://mail-archives.apache.org/mod_mbox/commons-dev/200408.mbox/%3c4122F3C1.9090402@tanukisoftware.com%3e
+    public void testParseFieldsOnFileJapaneseYear() {
+        final FTPFile f = getParser().parseFTPEntry(
+                "-rwxr-xr-x 2 user group 4096 3\u6708 2\u65e5 2003\u5e74 \u8a66\u9a13\u30d5\u30a1\u30a4\u30eb.csv");
+        assertNotNull("Could not parse entry.", f);
+        assertTrue("Should have been a file.", f.isFile());
+        checkPermissions(f);
+        assertEquals(2, f.getHardLinkCount());
+        assertEquals("user", f.getUser());
+        assertEquals("group", f.getGroup());
+        assertEquals("\u8a66\u9a13\u30d5\u30a1\u30a4\u30eb.csv", f.getName());
+        assertEquals(4096, f.getSize());
+
+        assertNotNull("Timestamp not null", f.getTimestamp());
+        final Calendar cal = Calendar.getInstance();
+        cal.set(Calendar.YEAR, 2003);
+        cal.set(Calendar.MONTH, Calendar.MARCH);
+        cal.set(Calendar.DATE, 2);
+        cal.set(Calendar.HOUR_OF_DAY, 0);
+        cal.set(Calendar.MINUTE, 0);
+        cal.set(Calendar.SECOND, 0);
+        assertEquals(df.format(cal.getTime()), df.format(f.getTimestamp().getTime()));
+    }
+
     @Override
-    protected void doAdditionalGoodTests(String test, FTPFile f) {
-        String link = f.getLink();
+    protected void doAdditionalGoodTests(final String test, final FTPFile f) {
+        final String link = f.getLink();
         if (null != link) {
-            int linklen = link.length();
+            final int linklen = link.length();
             if (linklen > 0) {
                 assertEquals(link, test.substring(test.length() - linklen));
                 assertEquals(f.getType(), FTPFile.SYMBOLIC_LINK_TYPE);
             }
         }
-        int type = f.getType();
+        final int type = f.getType();
         switch (test.charAt(0))
         {
         case 'd':
@@ -365,8 +418,8 @@ public class UnixFTPEntryParserTest extends FTPParseTestFramework {
             for (int perm = FTPFile.READ_PERMISSION;
                 perm <= FTPFile.EXECUTE_PERMISSION; perm++)
             {
-                int pos = 3*access + perm + 1;
-                char permchar = test.charAt(pos);
+                final int pos = 3*access + perm + 1;
+                final char permchar = test.charAt(pos);
                 assertEquals("Permission " + test.substring(1,10),
                         Boolean.valueOf(f.hasPermission(access, perm)),
                         Boolean.valueOf(permchar != '-' && !Character.isUpperCase(permchar)));
