@@ -15,20 +15,18 @@
  * limitations under the License.
  */
 package org.apache.commons.net.telnet;
-import junit.framework.TestCase;
-
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 
-/***
+import junit.framework.TestCase;
+
+/**
  * JUnit test class for TelnetClient.s
  * Implements protocol compliance tests
- * <p>
- * @author Bruno D'Avanzo
- ***/
+ */
 public class TelnetClientTest
 extends TestCase implements TelnetNotificationHandler
 {
@@ -41,9 +39,9 @@ extends TestCase implements TelnetNotificationHandler
         private final TelnetClient client;
         private final int port;
         TestConnection(
-                TelnetTestSimpleServer server,
-                TelnetClient client,
-                int port)
+                final TelnetTestSimpleServer server,
+                final TelnetClient client,
+                final int port)
         {
             this.server = server;
             this.client = client;
@@ -61,21 +59,36 @@ extends TestCase implements TelnetNotificationHandler
     private TestConnection OPTIONS;
     private TestConnection ANSI;
     private TestConnection NOREAD;
+    private TestConnection SMALL_BUFFER;
 
-    private final int NUM_CONNECTIONS = 4;
+    private final int NUM_CONNECTIONS = 5;
 
 
-    protected int numdo = 0;
-    protected int numdont = 0;
-    protected int numwill = 0;
-    protected int numwont = 0;
+    protected int numdo;
+    protected int numdont;
+    protected int numwill;
+    protected int numwont;
 
-    /***
+    protected int[] lastSubnegotiation;
+    protected int lastSubnegotiationLength;
+
+    /*
      * open connections needed for the tests for the test.
-     ***/
+     */
     @Override
     protected void setUp() throws Exception
     {
+        final SimpleOptionHandler subnegotiationSizeHandler = new SimpleOptionHandler(99, false, false, true, false)
+        {
+            @Override
+            public int[] answerSubnegotiation(final int[] suboptionData, final int suboptionLength)
+            {
+                lastSubnegotiation = suboptionData;
+                lastSubnegotiationLength = suboptionLength;
+                return null;
+            }
+        };
+
         int socket = 0;
         super.setUp();
         for (int port = 3333; socket < NUM_CONNECTIONS && port < 4000; port++)
@@ -89,16 +102,17 @@ extends TestCase implements TelnetNotificationHandler
                         client = new TelnetClient();
                         // redundant but makes code clearer.
                         client.setReaderThread(true);
+                        client.addOptionHandler(subnegotiationSizeHandler);
                         client.connect("127.0.0.1", port);
                         STANDARD = new TestConnection(server, client, port);
                         break;
                     case 1:
                         client = new TelnetClient();
-                        TerminalTypeOptionHandler ttopt =
+                        final TerminalTypeOptionHandler ttopt =
                             new TerminalTypeOptionHandler("VT100", false, false, true, false);
-                        EchoOptionHandler echoopt =
+                        final EchoOptionHandler echoopt =
                             new EchoOptionHandler(true, false, true, false);
-                        SuppressGAOptionHandler gaopt =
+                        final SuppressGAOptionHandler gaopt =
                             new SuppressGAOptionHandler(true, true, true, true);
 
                         client.addOptionHandler(ttopt);
@@ -118,10 +132,16 @@ extends TestCase implements TelnetNotificationHandler
                         client.connect("127.0.0.1", port);
                         NOREAD = new TestConnection(server, client, port);
                         break;
+                    case 4:
+                        client = new TelnetClient(8);
+                        client.addOptionHandler(subnegotiationSizeHandler);
+                        client.connect("127.0.0.1", port);
+                        SMALL_BUFFER = new TestConnection(server, client, port);
+                        break;
                }
                // only increment socket number on success
                socket++;
-           } catch (IOException e) {
+           } catch (final IOException e) {
                closeConnection(server, client, port);
            }
        }
@@ -140,9 +160,10 @@ extends TestCase implements TelnetNotificationHandler
         ANSI.close();
         OPTIONS.close();
         STANDARD.close();
+        SMALL_BUFFER.close();
         try {
             Thread.sleep(1000);
-        } catch (InterruptedException ie) {
+        } catch (final InterruptedException ie) {
             //do nothing
         }
         super.tearDown();
@@ -150,7 +171,7 @@ extends TestCase implements TelnetNotificationHandler
 
 
 
-    void closeConnection(TelnetTestSimpleServer server, TelnetClient client, int port) {
+    void closeConnection(final TelnetTestSimpleServer server, final TelnetClient client, final int port) {
         if (server != null) {
             server.disconnect();
             server.stop();
@@ -159,16 +180,16 @@ extends TestCase implements TelnetNotificationHandler
             if (client != null) {
                 client.disconnect();
             }
-        } catch (IOException e) {
+        } catch (final IOException e) {
             System.err.println("failed to close client-server connection on port " + port);
             System.err.println("ERROR in closeConnection(), "+ e.getMessage());
         }
 
     }
 
-    /***
+    /*
      * tests the initial condition of the sessions
-     ***/
+     */
     public void testInitial() throws Exception
     {
         boolean connect1_ok = false;
@@ -177,8 +198,8 @@ extends TestCase implements TelnetNotificationHandler
         boolean init2_ok = false;
         boolean add_invalid_ok1 = false;
         boolean add_invalid_ok2 = false;
-        byte buffread2[] = new byte[9];
-        byte expected2[] =
+        final byte buffread2[] = new byte[9];
+        final byte expected2[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WILL,
             (byte) TelnetOption.ECHO,
@@ -188,12 +209,12 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetOption.SUPPRESS_GO_AHEAD,
         };
 
-        SimpleOptionHandler hand = new SimpleOptionHandler(550);
+        final SimpleOptionHandler hand = new SimpleOptionHandler(550);
         try
         {
             STANDARD.client.addOptionHandler(hand);
         }
-        catch (Exception e)
+        catch (final Exception e)
         {
             add_invalid_ok1 = true;
         }
@@ -202,12 +223,12 @@ extends TestCase implements TelnetNotificationHandler
         {
             OPTIONS.client.addOptionHandler(hand);
         }
-        catch (Exception e)
+        catch (final Exception e)
         {
             add_invalid_ok2 = true;
         }
 
-        InputStream is1 = STANDARD.server.getInputStream();
+        final InputStream is1 = STANDARD.server.getInputStream();
         Thread.sleep(1000);
         if(is1.available() == 0)
         {
@@ -215,7 +236,7 @@ extends TestCase implements TelnetNotificationHandler
         }
 
         Thread.sleep(1000);
-        InputStream is2 = OPTIONS.server.getInputStream();
+        final InputStream is2 = OPTIONS.server.getInputStream();
         if(is2.available() == 9)
         {
             is2.read(buffread2);
@@ -226,7 +247,7 @@ extends TestCase implements TelnetNotificationHandler
             }
         }
 
-        InputStream is3 = ANSI.server.getInputStream();
+        final InputStream is3 = ANSI.server.getInputStream();
         Thread.sleep(1000);
         if(is3.available() == 0)
         {
@@ -248,27 +269,27 @@ extends TestCase implements TelnetNotificationHandler
         assertTrue(add_invalid_ok2);
     }
 
-    /***
+    /*
      * protocol compliance test for option negotiation
-     ***/
+     */
     public void testOptionNegotiation() throws Exception
     {
         boolean negotiation1_ok = false;
-        byte buffread1[] = new byte[6];
-        byte send1[] =
+        final byte buffread1[] = new byte[6];
+        final byte send1[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO, (byte) 15,
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WILL, (byte) 15,
         };
-        byte expected1[] =
+        final byte expected1[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT, (byte) 15,
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DONT, (byte) 15,
         };
 
         boolean negotiation2_ok = false;
-        byte buffread2[] = new byte[9];
-        byte send2[] =
+        final byte buffread2[] = new byte[9];
+        final byte send2[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO,
             (byte) TelnetOption.TERMINAL_TYPE,
@@ -279,7 +300,7 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT,
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
-        byte expected2[] =
+        final byte expected2[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WILL,
             (byte) TelnetOption.TERMINAL_TYPE,
@@ -289,14 +310,14 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
 
-        byte buffread2b[] = new byte[11];
-        byte send2b[] =
+        final byte buffread2b[] = new byte[11];
+        final byte send2b[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.SB,
             (byte) TelnetOption.TERMINAL_TYPE,
             (byte) 1, (byte) TelnetCommand.IAC, (byte) TelnetCommand.SE,
         };
-        byte expected2b[] =
+        final byte expected2b[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.SB,
             (byte) TelnetOption.TERMINAL_TYPE,
@@ -306,29 +327,29 @@ extends TestCase implements TelnetNotificationHandler
         };
 
         boolean negotiation3_ok = false;
-        byte buffread3[] = new byte[6];
-        byte send3[] =
+        final byte buffread3[] = new byte[6];
+        final byte send3[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO,
             (byte) TelnetOption.TERMINAL_TYPE,
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO,
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
-        byte expected3[] =
+        final byte expected3[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WILL,
             (byte) TelnetOption.TERMINAL_TYPE,
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT,
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
-        byte buffread3b[] = new byte[10];
-        byte send3b[] =
+        final byte buffread3b[] = new byte[10];
+        final byte send3b[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.SB,
             (byte) TelnetOption.TERMINAL_TYPE,
             (byte) 1, (byte) TelnetCommand.IAC, (byte) TelnetCommand.SE,
         };
-        byte expected3b[] =
+        final byte expected3b[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.SB,
             (byte) TelnetOption.TERMINAL_TYPE,
@@ -337,8 +358,8 @@ extends TestCase implements TelnetNotificationHandler
         };
 
 
-        InputStream is1 = STANDARD.server.getInputStream();
-        OutputStream os1 = STANDARD.server.getOutputStream();
+        final InputStream is1 = STANDARD.server.getInputStream();
+        final OutputStream os1 = STANDARD.server.getOutputStream();
         is1.skip(is1.available());
         os1.write(send1);
         os1.flush();
@@ -352,8 +373,8 @@ extends TestCase implements TelnetNotificationHandler
             }
         }
 
-        InputStream is2 = OPTIONS.server.getInputStream();
-        OutputStream os2 = OPTIONS.server.getOutputStream();
+        final InputStream is2 = OPTIONS.server.getInputStream();
+        final OutputStream os2 = OPTIONS.server.getOutputStream();
         Thread.sleep(1000);
         is2.skip(is2.available());
         os2.write(send2);
@@ -384,8 +405,8 @@ extends TestCase implements TelnetNotificationHandler
             }
         }
 
-        InputStream is3 = ANSI.server.getInputStream();
-        OutputStream os3 = ANSI.server.getOutputStream();
+        final InputStream is3 = ANSI.server.getInputStream();
+        final OutputStream os3 = ANSI.server.getOutputStream();
         Thread.sleep(1000);
         is3.skip(is3.available());
         os3.write(send3);
@@ -431,15 +452,15 @@ extends TestCase implements TelnetNotificationHandler
     }
 
 
-    /***
+    /*
      * protocol compliance test for option renegotiation
-     ***/
+     */
     public void testOptionRenegotiation() throws Exception
     {
         boolean negotiation1_ok = false;
 
-        byte buffread[] = new byte[6];
-        byte send[] =
+        final byte buffread[] = new byte[6];
+        final byte send[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO,
             (byte) TelnetOption.ECHO,
@@ -448,7 +469,7 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT,
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
-        byte expected[] =
+        final byte expected[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT,
             (byte) TelnetOption.SUPPRESS_GO_AHEAD,
@@ -456,21 +477,21 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
 
-        byte buffread2[] = new byte[3];
-        byte send2[] =
+        final byte buffread2[] = new byte[3];
+        final byte send2[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DONT,
             (byte) TelnetOption.ECHO,
         };
-        byte expected2[] =
+        final byte expected2[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT,
             (byte) TelnetOption.ECHO,
         };
 
 
-        InputStream is = OPTIONS.server.getInputStream();
-        OutputStream os = OPTIONS.server.getOutputStream();
+        final InputStream is = OPTIONS.server.getInputStream();
+        final OutputStream os = OPTIONS.server.getOutputStream();
         Thread.sleep(1000);
         is.skip(is.available());
         os.write(send);
@@ -504,20 +525,20 @@ extends TestCase implements TelnetNotificationHandler
         assertTrue(!OPTIONS.client.getLocalOptionState(TelnetOption.ECHO));
     }
 
-    /***
+    /*
      * test of option negotiation notification
-     ***/
+     */
     public void testNotification() throws Exception
     {
-        byte buffread1[] = new byte[6];
-        byte send1[] =
+        final byte buffread1[] = new byte[6];
+        final byte send1[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO, (byte) 15,
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WILL, (byte) 15,
         };
 
-        byte buffread2[] = new byte[9];
-        byte send2[] =
+        final byte buffread2[] = new byte[9];
+        final byte send2[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO,
             (byte) TelnetOption.TERMINAL_TYPE,
@@ -529,7 +550,7 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
 
-        byte buffread2b[] = new byte[11];
+        final byte buffread2b[] = new byte[11];
 
 
         numdo = 0;
@@ -538,8 +559,8 @@ extends TestCase implements TelnetNotificationHandler
         numwont = 0;
         OPTIONS.client.registerNotifHandler(this);
 
-        InputStream is1 = STANDARD.server.getInputStream();
-        OutputStream os1 = STANDARD.server.getOutputStream();
+        final InputStream is1 = STANDARD.server.getInputStream();
+        final OutputStream os1 = STANDARD.server.getOutputStream();
         is1.skip(is1.available());
         os1.write(send1);
         os1.flush();
@@ -549,8 +570,8 @@ extends TestCase implements TelnetNotificationHandler
             is1.read(buffread1);
         }
 
-        InputStream is2 = OPTIONS.server.getInputStream();
-        OutputStream os2 = OPTIONS.server.getOutputStream();
+        final InputStream is2 = OPTIONS.server.getInputStream();
+        final OutputStream os2 = OPTIONS.server.getOutputStream();
         Thread.sleep(500);
         is2.skip(is2.available());
         os2.write(send2);
@@ -574,17 +595,17 @@ extends TestCase implements TelnetNotificationHandler
     }
 
 
-    /***
+    /*
      * protocol compliance test in case of option handler removal
-     ***/
+     */
     public void testDeleteOptionHandler() throws Exception
     {
         boolean remove_ok = false;
         boolean remove_invalid_ok1 = false;
         boolean remove_invalid_ok2 = false;
 
-        byte buffread[] = new byte[6];
-        byte send[] =
+        final byte buffread[] = new byte[6];
+        final byte send[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO,
             (byte) TelnetOption.ECHO,
@@ -594,7 +615,7 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
 
-        byte expected[] =
+        final byte expected[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT,
             (byte) TelnetOption.SUPPRESS_GO_AHEAD,
@@ -602,8 +623,8 @@ extends TestCase implements TelnetNotificationHandler
             (byte) TelnetOption.SUPPRESS_GO_AHEAD
         };
 
-        InputStream is = OPTIONS.server.getInputStream();
-        OutputStream os = OPTIONS.server.getOutputStream();
+        final InputStream is = OPTIONS.server.getInputStream();
+        final OutputStream os = OPTIONS.server.getOutputStream();
         Thread.sleep(1000);
         is.skip(is.available());
         os.write(send);
@@ -626,7 +647,7 @@ extends TestCase implements TelnetNotificationHandler
         {
             OPTIONS.client.deleteOptionHandler(TelnetOption.SUPPRESS_GO_AHEAD);
         }
-        catch (Exception e)
+        catch (final Exception e)
         {
             remove_invalid_ok1 = true;
         }
@@ -635,7 +656,7 @@ extends TestCase implements TelnetNotificationHandler
         {
             OPTIONS.client.deleteOptionHandler(550);
         }
-        catch (Exception e)
+        catch (final Exception e)
         {
             remove_invalid_ok2 = true;
         }
@@ -649,30 +670,30 @@ extends TestCase implements TelnetNotificationHandler
     }
 
 
-    /***
+    /*
      * test of AYT functionality
-     ***/
+     */
     public void testAYT() throws Exception
     {
         boolean ayt_true_ok = false;
         boolean ayt_false_ok = false;
 
 
-        byte AYT[] = { (byte) TelnetCommand.IAC, (byte) TelnetCommand.AYT };
-        byte response[] =
+        final byte AYT[] = { (byte) TelnetCommand.IAC, (byte) TelnetCommand.AYT };
+        final byte response[] =
             { (byte) '[', (byte) 'Y', (byte) 'e', (byte) 's', (byte) ']' };
-        String inputs[] = new String[1];
-        String outputs[] = new String[1];
+        final String inputs[] = new String[1];
+        final String outputs[] = new String[1];
         inputs[0] = new String (AYT);
         outputs[0] = new String (response);
 
 
-        OutputStream os = ANSI.server.getOutputStream();
-        InputStream is = ANSI.server.getInputStream();
-        TelnetTestResponder tr =
+        final OutputStream os = ANSI.server.getOutputStream();
+        final InputStream is = ANSI.server.getInputStream();
+        final TelnetTestResponder tr =
             new TelnetTestResponder(is, os, inputs, outputs, 30000);
         assertNotNull(tr);
-        boolean res1 = ANSI.client.sendAYT(2000);
+        final boolean res1 = ANSI.client.sendAYT(2000);
 
         if (res1 == true) {
             ayt_true_ok=true;
@@ -681,7 +702,7 @@ extends TestCase implements TelnetNotificationHandler
         Thread.sleep(1000);
         is.skip(is.available());
 
-        boolean res2 = ANSI.client.sendAYT(2000);
+        final boolean res2 = ANSI.client.sendAYT(2000);
 
         if (res2 == false) {
             ayt_false_ok=true;
@@ -692,108 +713,100 @@ extends TestCase implements TelnetNotificationHandler
         assertTrue(ayt_false_ok);
     }
 
-    /***
+    /*
      * test of Spy functionality
-     ***/
+     */
     public void testSpy() throws Exception
     {
         boolean test1spy_ok = false;
         boolean test2spy_ok = false;
         boolean stopspy_ok = false;
-        byte expected1[] =
-            { (byte) 't', (byte) 'e', (byte) 's', (byte) 't', (byte) '1' };
-        byte expected2[] =
-            { (byte) 't', (byte) 'e', (byte) 's', (byte) 't', (byte) '2' };
+        final byte expected1[] = {(byte) 't', (byte) 'e', (byte) 's', (byte) 't', (byte) '1'};
+        final byte expected2[] = {(byte) 't', (byte) 'e', (byte) 's', (byte) 't', (byte) '2'};
 
+        try (final PipedOutputStream po = new PipedOutputStream();
+            final PipedInputStream pi = new PipedInputStream(po)) {
 
-        PipedOutputStream po = new PipedOutputStream();
-        PipedInputStream pi = new PipedInputStream(po);
+            final OutputStream os = STANDARD.server.getOutputStream();
+            final OutputStream ostc = STANDARD.client.getOutputStream();
 
-        OutputStream os = STANDARD.server.getOutputStream();
-        OutputStream ostc = STANDARD.client.getOutputStream();
+            STANDARD.client.registerSpyStream(po);
 
-        STANDARD.client.registerSpyStream(po);
+            os.write("test1".getBytes());
+            os.flush();
 
-        os.write("test1".getBytes());
-        os.flush();
+            Thread.sleep(1000);
+            final byte buffer[] = new byte[5];
 
-        Thread.sleep(1000);
-        byte buffer[] = new byte[5];
-
-        if(pi.available() == 5)
-        {
-            pi.read(buffer);
-            if (equalBytes(buffer, expected1)) {
-                test1spy_ok = true;
+            if (pi.available() == 5) {
+                pi.read(buffer);
+                if (equalBytes(buffer, expected1)) {
+                    test1spy_ok = true;
+                }
             }
-        }
 
-        ostc.write("test2".getBytes());
-        ostc.flush();
+            ostc.write("test2".getBytes());
+            ostc.flush();
 
-        Thread.sleep(1000);
+            Thread.sleep(1000);
 
-        if(pi.available() == 5)
-        {
-            pi.read(buffer);
-            if (equalBytes(buffer, expected2)) {
-                test2spy_ok = true;
+            if (pi.available() == 5) {
+                pi.read(buffer);
+                if (equalBytes(buffer, expected2)) {
+                    test2spy_ok = true;
+                }
             }
+
+            STANDARD.client.stopSpyStream();
+            os.write("test1".getBytes());
+            os.flush();
+            ostc.write("test2".getBytes());
+            ostc.flush();
+            Thread.sleep(1000);
+            if (pi.available() == 0) {
+                stopspy_ok = true;
+            }
+
+            assertTrue(test1spy_ok);
+            assertTrue(test2spy_ok);
+            assertTrue(stopspy_ok);
         }
-
-        STANDARD.client.stopSpyStream();
-        os.write("test1".getBytes());
-        os.flush();
-        ostc.write("test2".getBytes());
-        ostc.flush();
-        Thread.sleep(1000);
-        if(pi.available() == 0)
-        {
-            stopspy_ok = true;
-        }
-
-
-        assertTrue(test1spy_ok);
-        assertTrue(test2spy_ok);
-        assertTrue(stopspy_ok);
-        pi.close();
     }
 
-    /***
+    /*
      * test of setReaderThread
-     ***/
+     */
     public void testSetReaderThread() throws Exception
     {
         boolean negotiation1_ok = false;
         boolean negotiation2_ok = false;
         boolean read_ok = false;
-        byte buffread1[] = new byte[6];
-        byte send1[] =
+        final byte buffread1[] = new byte[6];
+        final byte send1[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DO, (byte) 15,
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WILL, (byte) 15,
         };
-        byte expected1[] =
+        final byte expected1[] =
         {
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.WONT, (byte) 15,
             (byte) TelnetCommand.IAC, (byte) TelnetCommand.DONT, (byte) 15,
         };
 
 
-        InputStream is1 = NOREAD.server.getInputStream();
-        OutputStream os1 = NOREAD.server.getOutputStream();
+        final InputStream is1 = NOREAD.server.getInputStream();
+        final OutputStream os1 = NOREAD.server.getOutputStream();
         is1.skip(is1.available());
         os1.write(send1);
         os1.flush();
         os1.write("A".getBytes());
         os1.flush();
         Thread.sleep(1000);
-        InputStream instr = NOREAD.client.getInputStream();
-        byte[] buff = new byte[4];
-        int ret_read = 0;
+        final InputStream instr = NOREAD.client.getInputStream();
+        final byte[] buff = new byte[4];
 
-        ret_read = instr.read(buff);
-        if((ret_read == 1) && (buff[0] == 'A'))
+        final int ret_read = instr.read(buff);
+        if(ret_read == 1 && buff[0] == 'A')
         {
             read_ok = true;
         }
@@ -816,8 +829,8 @@ extends TestCase implements TelnetNotificationHandler
             }
 
 
-        InputStream is2 = STANDARD.server.getInputStream();
-        OutputStream os2 = STANDARD.server.getOutputStream();
+        final InputStream is2 = STANDARD.server.getInputStream();
+        final OutputStream os2 = STANDARD.server.getOutputStream();
         Thread.sleep(1000);
         is2.skip(is2.available());
         os2.write(send1);
@@ -846,31 +859,79 @@ extends TestCase implements TelnetNotificationHandler
         assertTrue("Expected negotiation2_ok to be true, got " + negotiation2_ok, negotiation2_ok);
     }
 
+    /*
+     * test of max subnegotiation length
+     */
+    public void testMaxSubnegotiationLength() throws Exception
+    {
+        final byte send[] =
+            {
+                (byte) TelnetCommand.IAC, (byte) TelnetCommand.SB, (byte) 99,
+                (byte) 1, (byte) 2, (byte) 3,
+                (byte) 4, (byte) 5, (byte) 6,
+                (byte) 7, (byte) 8, (byte) 9,
+                (byte) 10, (byte) 11, (byte) 12,
+                (byte) 13, (byte) 14, (byte) 15,
+                (byte) TelnetCommand.IAC, (byte) TelnetCommand.SE,
+            };
 
-    /***
+        final OutputStream os1 = SMALL_BUFFER.server.getOutputStream();
+        os1.write(send);
+        os1.flush();
+        Thread.sleep(500);
+
+        // we sent 16 bytes, but the buffer size should just be 8
+        assertEquals(8, lastSubnegotiationLength);
+        assertEquals(8, lastSubnegotiation.length);
+        assertEquals(99, lastSubnegotiation[0]);
+        assertEquals(1, lastSubnegotiation[1]);
+        assertEquals(2, lastSubnegotiation[2]);
+        assertEquals(3, lastSubnegotiation[3]);
+        assertEquals(4, lastSubnegotiation[4]);
+        assertEquals(5, lastSubnegotiation[5]);
+        assertEquals(6, lastSubnegotiation[6]);
+        assertEquals(7, lastSubnegotiation[7]);
+
+        final OutputStream os2 = STANDARD.server.getOutputStream();
+        os2.write(send);
+        os2.flush();
+        Thread.sleep(500);
+
+        // the standard subnegotiation buffer size is 512
+        assertEquals(16, lastSubnegotiationLength);
+        assertEquals(512, lastSubnegotiation.length);
+        assertEquals(99, lastSubnegotiation[0]);
+        assertEquals(1, lastSubnegotiation[1]);
+        assertEquals(2, lastSubnegotiation[2]);
+        assertEquals(3, lastSubnegotiation[3]);
+        assertEquals(4, lastSubnegotiation[4]);
+        assertEquals(5, lastSubnegotiation[5]);
+        assertEquals(6, lastSubnegotiation[6]);
+        assertEquals(7, lastSubnegotiation[7]);
+    }
+
+
+    /*
      * Helper method. compares two arrays of int
-     ***/
-    protected boolean equalBytes(byte a1[], byte a2[])
+     */
+    protected boolean equalBytes(final byte a1[], final byte a2[])
     {
         if(a1.length != a2.length)
         {
-            return(false);
+            return false;
         }
-        else
+        boolean result = true;
+        for(int ii=0; ii<a1.length; ii++)
         {
-            boolean result = true;
-            for(int ii=0; ii<a1.length; ii++)
-            {
 
-                if (a1[ii]!= a2[ii]) {
-                    result = false;
-                }
+            if (a1[ii]!= a2[ii]) {
+                result = false;
             }
-            return(result);
         }
+        return result;
     }
 
-    /***
+    /*
      * Callback method called when TelnetClient receives an option
      * negotiation command.
      * <p>
@@ -879,9 +940,9 @@ extends TestCase implements TelnetNotificationHandler
      * <p>
      * @param option_code - code of the option negotiated
      * <p>
-     ***/
-//    @Override
-    public void receivedNegotiation(int negotiation_code, int option_code)
+     */
+    @Override
+    public void receivedNegotiation(final int negotiation_code, final int option_code)
     {
         if(negotiation_code == TelnetNotificationHandler.RECEIVED_DO)
         {
